@@ -15,18 +15,34 @@ setClass("DataFrame",
          prototype(rownames = NULL,
                    nrows = 0L,
                    listData = structure(list(), names = character())),
-         contains = c("DataTable", "SimpleList"))
+         contains = c("RectangularData", "SimpleList"))
+
+### Add DataFrame to the DataFrame_OR_NULL union.
+setIs("DataFrame", "DataFrame_OR_NULL")
 
 ## Just a direct DataFrame extension with no additional slot for now. Once all
 ## serialized DataFrame instances are replaced with DFrame instances (which
 ## will take several BioC release cycles) we'll be able to move the DataFrame
 ## slots from the DataFrame class definition to the DFrame class definition.
 ## The final goal is to have DataFrame become a virtual class with no slots
-## that only extends DataTable, and DFrame a concrete DataFrame and SimpleList
-## subclass that has the same slots as the current DataFrame class.
+## that only extends RectangularData, and DFrame a concrete DataFrame and
+## SimpleList subclass that has the same slots as the current DataFrame class.
 setClass("DFrame", contains="DataFrame")
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### vertical_slot_names() and horizontal_slot_names()
+###
+
+setMethod("vertical_slot_names", "DataFrame",
+    function(x) "rownames"
+)
+
+setMethod("horizontal_slot_names", "DataFrame",
+    function(x) parallel_slot_names(x)
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### updateObject()
 ###
@@ -40,7 +56,7 @@ setMethod("updateObject", "DataFrame",
             ## to be replaced with DFrame instances. Note that this is NOT a
             ## change of the internals, only a change of the class attribute.
             if (verbose)
-                message("[updateObject] Settting class attribute of DataFrame ",
+                message("[updateObject] Setting class attribute of DataFrame ",
                         "instance to \"DFrame\" ... ", appendLF=FALSE)
             class(object) <- class(new("DFrame"))
             if (verbose)
@@ -65,6 +81,8 @@ setMethod("nrow", "DataFrame", function(x) x@nrows)
 
 setMethod("ncol", "DataFrame", function(x) length(x))
 
+setMethod("dim", "DataFrame", function(x) c(nrow(x), ncol(x)))
+
 setMethod("rownames", "DataFrame",
           function(x, do.NULL = TRUE, prefix = "row")
           {
@@ -93,6 +111,10 @@ setMethod("colnames", "DataFrame",
             return(character(0))
           })
 
+setMethod("dimnames", "DataFrame",
+    function(x) list(rownames(x), colnames(x))
+)
+
 setReplaceMethod("rownames", "DataFrame",
                  function(x, value)
                  {
@@ -120,6 +142,17 @@ setReplaceMethod("colnames", "DataFrame",
                    x
                  })
 
+setReplaceMethod("dimnames", "DataFrame",
+    function(x, value)
+    {
+        if (!(is.list(value) && length(value) == 2L))
+            stop("dimnames replacement value must be a list of length 2")
+        rownames(x) <- value[[1L]]
+        colnames(x) <- value[[2L]]
+        x
+    }
+)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Validity
@@ -621,12 +654,6 @@ setMethod("rep", "DataFrame", function(x, ...) {
 ### Coercion
 ###
 
-## Break DataFrame into a normal R data.frame
-setAs("DataFrame", "data.frame",
-      function(from) {
-        as.data.frame(from, optional=TRUE)
-      })
-
 .as.data.frame.DataFrame <- function(x, row.names=NULL, optional=FALSE,
                                      stringsAsFactors=FALSE, ...)
 {
@@ -781,7 +808,7 @@ setAs("AsIs", "DFrame",
         new_DataFrame(setNames(list(drop_AsIs(from)), "X"))
       })
 
-setAs("ANY", "DataTable_OR_NULL", function(from) as(from, "DFrame"))
+setAs("ANY", "DataFrame_OR_NULL", function(from) as(from, "DFrame"))
 
 setMethod("coerce2", "DataFrame",
     function(from, to)
@@ -839,6 +866,51 @@ setMethod("classNameForDisplay", "DFrame",
     function(x) if (class(x) == "DFrame") "DataFrame" else class(x)
 )
 
+setMethod("makeNakedCharacterMatrixForDisplay", "DataFrame",
+    function(x)
+    {
+        df <- as.data.frame(lapply(x, showAsCell), optional=TRUE)
+        as.matrix(format(df))
+    }
+)
+
+make_class_info_for_DataFrame_display <- function(x)
+{
+    vapply(x, function(xi) paste0("<", classNameForDisplay(xi), ">"),
+           character(1), USE.NAMES=FALSE)
+}
+
+.show_DataFrame <- function(x)
+{
+    nhead <- get_showHeadLines()
+    ntail <- get_showTailLines()
+    x_nrow <- nrow(x)
+    x_ncol <- ncol(x)
+    cat(classNameForDisplay(x), " with ",
+        x_nrow, " row", ifelse(x_nrow == 1L, "", "s"),
+        " and ",
+        x_ncol, " column", ifelse(x_ncol == 1L, "", "s"),
+        "\n", sep="")
+    if (x_nrow != 0L && x_ncol != 0L) {
+        x_rownames <- rownames(x)
+        if (x_nrow <= nhead + ntail + 1L) {
+            m <- makeNakedCharacterMatrixForDisplay(x)
+            if (!is.null(x_rownames))
+                rownames(m) <- x_rownames
+        } else {
+            m <- rbind(makeNakedCharacterMatrixForDisplay(head(x, nhead)),
+                       rbind(rep.int("...", x_ncol)),
+                       makeNakedCharacterMatrixForDisplay(tail(x, ntail)))
+            rownames(m) <- make_rownames_for_RectangularData_display(
+                                             x_rownames, x_nrow,
+                                             nhead, ntail)
+        }
+        m <- rbind(make_class_info_for_DataFrame_display(x), m)
+        print(m, quote=FALSE, right=TRUE)
+    }
+    invisible(NULL)
+}
+
 setMethod("show", "DataFrame",
     function(object)
     {
@@ -847,7 +919,7 @@ setMethod("show", "DataFrame",
             #warning(wmsg(.OLD_DATAFRAME_INSTANCE_MSG))
             object <- updateObject(object, check=FALSE)
         }
-        invisible(callNextMethod())
+        .show_DataFrame(object)
     }
 )
 
