@@ -36,13 +36,13 @@ setClass("SortedByQuerySelfHits", contains=c("SelfHits", "SortedByQueryHits"))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### vertical_slot_names()
+### parallel_slot_names()
 ###
 
-### Combine the new "vertical slots" with those of the parent class. Make
-### sure to put the new vertical slots **first**. See bindROWS.R file for
-### what slots should or should not be considered "vertical".
-setMethod("vertical_slot_names", "Hits",
+### Combine the new "parallel slots" with those of the parent class. Make
+### sure to put the new parallel slots **first**. See Vector-class.R file
+### for what slots should or should not be considered "parallel".
+setMethod("parallel_slot_names", "Hits",
     function(x) c("from", "to", callNextMethod())
 )
 
@@ -343,6 +343,29 @@ setMethod("as.matrix", "Hits",
 
 setMethod("as.table", "Hits", .count_Lnode_hits)
 
+### FIXME: Coercions of Vector derivatives to DFrame are inconsistent.
+### For some Vector derivatives (e.g. IRanges, GRanges) the object is stored
+### "as is" in the 1st column of the returned DFrame, whereas for others (e.g.
+### Hits below) the object is "dismantled" into various parallel components
+### that end up in separate columns of the returned DFrame.
+setAs("Hits", "DFrame",
+    function(from)
+    {
+        from_mcols <- mcols(from, use.names=FALSE)
+        if (is.null(from_mcols))
+            from_mcols <- make_zero_col_DFrame(length(from))
+        DataFrame(as.matrix(from), from_mcols, check.names=FALSE)
+    }
+)
+
+### S3/S4 combo for as.data.frame.Hits
+as.data.frame.Hits <- function(x, row.names=NULL, optional=FALSE, ...)
+{
+    x <- as(x, "DFrame")
+    as.data.frame(x, row.names=row.names, optional=optional, ...)
+}
+setMethod("as.data.frame", "Hits", as.data.frame.Hits)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting
@@ -391,8 +414,8 @@ setMethod("summary", "Hits", summary.Hits)
 
 .from_Hits_to_naked_character_matrix_for_display <- function(x)
 {
-    m <- cbind(from=as.character(from(x)),
-               to=as.character(to(x)))
+    m <- cbind(from=showAsCell(from(x)),
+               to=showAsCell(to(x)))
     if (is(x, "SortedByQueryHits"))
         colnames(m) <- c("queryHits", "subjectHits")
     cbind_mcols_for_display(m, x)
