@@ -134,7 +134,6 @@ setAs("Rle", "character", function(from) as.character(from))
 setAs("Rle", "raw", function(from) as.raw(from))
 setAs("Rle", "factor", function(from) as.factor(from))
 setAs("Rle", "list", function(from) as.list(from))
-setAs("Rle", "data.frame", function(from) as.data.frame(from))
 
 as.vector.Rle <- function(x, mode)
   rep.int(as.vector(runValue(x), mode), runLength(x))
@@ -751,7 +750,8 @@ setMethod("isStrictlySorted", "Rle",
     function(x) all(runLength(x) == 1L) && isStrictlySorted(runValue(x))
 )
 
-.sort.Rle <- function(x, decreasing=FALSE, na.last=NA, ...)
+### S3/S4 combo for sort.Rle
+sort.Rle <- function(x, decreasing=FALSE, na.last=NA, ...)
 {
     if (is.na(na.last)) {
         if (anyNA(runValue(x)))
@@ -760,7 +760,7 @@ setMethod("isStrictlySorted", "Rle",
     ord <- base::order(runValue(x), na.last=na.last, decreasing=decreasing)
     new_Rle(runValue(x)[ord], runLength(x)[ord])
 }
-setMethod("sort", "Rle", .sort.Rle)
+setMethod("sort", "Rle", sort.Rle)
 
 setMethod("rank", "Rle", function (x, na.last = TRUE,
                                    ties.method = c("average", "first", 
@@ -897,14 +897,14 @@ setMethod("show", "Rle",
                   " with ", nr, ifelse(nr == 1, " run\n", " runs\n"), sep = "")
               first <- max(1L, halfWidth)
               showMatrix <-
-                rbind(as.character(head(runLength(object), first)),
-                      as.character(head(runValue(object), first)))
+                rbind(showAsCell(head(runLength(object), first)),
+                      showAsCell(head(runValue(object), first)))
               if (nr > first) {
                   last <- min(nr - first, halfWidth)
                   showMatrix <-
                     cbind(showMatrix,
-                          rbind(as.character(tail(runLength(object), last)),
-                                as.character(tail(runValue(object), last))))
+                          rbind(showAsCell(tail(runLength(object), last)),
+                                showAsCell(tail(runValue(object), last))))
               }
               if (is.character(runValue(object))) {
                   showMatrix[2L,] <-
@@ -917,5 +917,3 @@ setMethod("show", "Rle",
                   cat(labeledLine("Levels", levels(object)))
           })
 
-setMethod("showAsCell", "Rle", function(object) as.vector(object))
-
