@@ -12,13 +12,13 @@ setClass("SimpleList",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### vertical_slot_names()
+### parallel_slot_names()
 ###
 
-### Combine the new "vertical slots" with those of the parent class. Make
-### sure to put the new vertical slots **first**. See bindROWS.R file for
-### what slots should or should not be considered "vertical".
-setMethod("vertical_slot_names", "SimpleList",
+### Combine the new "parallel slots" with those of the parent class. Make
+### sure to put the new parallel slots **first**. See Vector-class.R file
+### for what slots should or should not be considered "parallel".
+setMethod("parallel_slot_names", "SimpleList",
     function(x) c("listData", callNextMethod())
 )
 
