@@ -6,13 +6,11 @@
 ### sequence i.e. an ordered finite collection of elements.
 ###
 
-setClassUnion("DataTable_OR_NULL", c("DataTable", "NULL"))
-
 setClass("Vector",
     contains="Annotated",
     representation(
         "VIRTUAL",
-        elementMetadata="DataTable_OR_NULL"
+        elementMetadata="DataFrame_OR_NULL"
     )
 )
 
@@ -31,17 +29,38 @@ setClassUnion("vector_OR_Vector", c("vector", "Vector"))  # vector-like objects
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### vertical_slot_names()
+### parallel_slot_names()
 ###
-### Methods for Vector subclasses should be defined in an incremental fashion,
-### that is, they should only explicitly list the new "vertical slots" (i.e.
-### the vertical slots that they introduce). See bindROWS.R file for what
-### slots should or should not be considered "vertical". See Hits-class.R
-### file for an example of a vertical_slot_names() method defined for a
-### Vector derivative.
+### For internal use only.
 ###
 
-setMethod("vertical_slot_names", "Vector", function(x) "elementMetadata")
+### parallel_slot_names() must return the names of all the slots in Vector
+### derivative 'x' that are **parallel** to the object. Slot "foo" in 'x'
+### is considered to be parallel to 'x' if it's guaranteed to contain a
+### value that is either NULL or such that 'NROW(x@foo)' is equal to
+### 'length(x)' and the i-th ROW in 'x@foo' is associated with the i-th
+### vector element in 'x'.
+### For example, the "start", "width", "NAMES", and "elementMetadata" slots
+### of an IRanges object 'x' are parallel to 'x'. Note that the "NAMES"
+### and "elementMetadata" slots can be set to NULL.
+setGeneric("parallel_slot_names",
+    function(x) standardGeneric("parallel_slot_names")
+)
+
+parallelSlotNames <- function(x)
+{
+    .Defunct("parallel_slot_names")
+    parallel_slot_names(x)
+}
+
+
+### Methods for Vector derivatives should be defined in an incremental
+### fashion, that is, they should only explicitly list the new "parallel
+### slots" (i.e. the parallel slots that they add to their parent class).
+### See above for what slots should or should not be considered "parallel".
+### See Hits-class.R file for an example of a parallel_slot_names() method
+### defined for a Vector derivative.
+setMethod("parallel_slot_names", "Vector", function(x) "elementMetadata")
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -84,12 +103,12 @@ setMethod("updateObject", "Vector",
 ### Getters
 ###
 
-### We use the **first** slot returned by vertical_slot_names() to infer
+### We use the **first** slot returned by parallel_slot_names() to infer
 ### the length of 'x' so hopefully this is not a slot that can contain a
 ### NULL (like the "elementMetadata" of a Vector derivative or the "NAMES"
 ### slot of an IRanges object).
 setMethod("length", "Vector",
-    function(x) NROW(slot(x, vertical_slot_names(x)[[1L]]))
+    function(x) NROW(slot(x, parallel_slot_names(x)[[1L]]))
 )
 
 setMethod("lengths", "Vector",
@@ -104,9 +123,6 @@ setMethod("lengths", "Vector",
      }
 )
 
-setMethod("NROW", "Vector", function(x) length(x))
-setMethod("ROWNAMES", "Vector", function(x) names(x))
-
 ### 3 accessors for the same slot: elementMetadata(), mcols(), and values().
 ### mcols() is the recommended one, use of elementMetadata() or values() is
 ### discouraged.
@@ -158,24 +174,24 @@ setMethod("is.na", "Vector", function(x) rep.int(FALSE, length(x)))
     NULL
 }
 
-.validate_Vector_vertical_slots <- function(x)
+.validate_Vector_parallel_slots <- function(x)
 {
     x_len <- length(x)
-    x_vslotnames <- vertical_slot_names(x)
-    if (!is.character(x_vslotnames)
-     || anyMissing(x_vslotnames)
-     || anyDuplicated(x_vslotnames)) {
-        msg <- c("'vertical_slot_names(x)' must be a character vector ",
+    x_pslotnames <- parallel_slot_names(x)
+    if (!is.character(x_pslotnames)
+     || anyMissing(x_pslotnames)
+     || anyDuplicated(x_pslotnames)) {
+        msg <- c("'parallel_slot_names(x)' must be a character vector ",
                  "with no NAs and no duplicates")
         return(paste(msg, collapse=""))
     }
-    if (x_vslotnames[[length(x_vslotnames)]] != "elementMetadata") {
-        msg <- c("last string in 'vertical_slot_names(x)' ",
+    if (x_pslotnames[[length(x_pslotnames)]] != "elementMetadata") {
+        msg <- c("last string in 'parallel_slot_names(x)' ",
                  "must be \"elementMetadata\"")
         return(paste(msg, collapse=""))
     }
     msg <- NULL
-    for (slotname in head(x_vslotnames, -1L)) {
+    for (slotname in head(x_pslotnames, -1L)) {
         tmp <- slot(x, slotname)
         if (!(is.null(tmp) || NROW(tmp) == x_len)) {
             what <- paste0("x@", slotname)
@@ -209,15 +225,15 @@ setMethod("is.na", "Vector", function(x) rep.int(FALSE, length(x)))
     x_mcols <- mcols(x, use.names=FALSE)
     if (is.null(x_mcols))
         return(NULL)
-    if (!is(x_mcols, "DataTable"))
-        return("'mcols(x)' must be a DataTable object or NULL")
-    ## 'x_mcols' is a DataTable object.
+    if (!is(x_mcols, "DataFrame"))
+        return("'mcols(x)' must be a DataFrame object or NULL")
+    ## 'x_mcols' is a DataFrame derivative.
     x_mcols_rownames <- rownames(x_mcols)
     if (is.null(x_mcols_rownames))
         return(NULL)
     if (!identical(x_mcols_rownames, names(x)))
     {
-        msg <- c("the rownames of DataTable 'mcols(x)' ",
+        msg <- c("the rownames of DataFrame 'mcols(x)' ",
                  "must match the names of 'x'")
         return(paste(msg, collapse=""))
     }
@@ -227,7 +243,7 @@ setMethod("is.na", "Vector", function(x) rep.int(FALSE, length(x)))
 .validate_Vector <- function(x)
 {
     c(.validate_Vector_length(x),
-      .validate_Vector_vertical_slots(x),
+      .validate_Vector_parallel_slots(x),
       .validate_Vector_names(x),
       .validate_Vector_mcols(x))
 }
@@ -267,7 +283,7 @@ normarg_mcols <- function(mcols, x_class, x_len)
     if (!ok)
         mcols <- as(mcols, mcols_target_class)
 
-    ## From now on, 'mcols' is guaranteed to be a DataTable object.
+    ## From now on, 'mcols' is guaranteed to be a DataFrame derivative.
     if (!is.null(rownames(mcols)))
         rownames(mcols) <- NULL
 
@@ -374,7 +390,7 @@ setAs("Vector", "raw", function(from) as.raw(from))
 
 setAs("Vector", "factor", function(from) as.factor(from))
 
-setAs("Vector", "data.frame", function(from) as.data.frame(from))
+setAs("Vector", "data.frame", function(from) as.data.frame(from, optional=TRUE))
 
 ### S3/S4 combo for as.data.frame.Vector
 as.data.frame.Vector <- function(x, row.names=NULL, optional=FALSE, ...) {
@@ -419,6 +435,17 @@ makeFixedColumnEnv <- function(x, parent, tform = identity) {
   env
 }
 
+setGeneric("as.env", function(x, ...) standardGeneric("as.env"))
+
+setMethod("as.env", "NULL", function(x, enclos, tform = identity) {
+  new.env(parent=enclos)
+})
+
+addSelfRef <- function(x, env) {
+  env$.. <- x
+  env
+}
+
 setMethod("as.env", "Vector", function(x, enclos, tform = identity) {
   parent <- as.env(mcols(x, use.names=FALSE), enclos, tform)
   addSelfRef(x, makeFixedColumnEnv(x, parent, tform))
@@ -455,13 +482,13 @@ setMethod("[", "Vector",
 )
 
 ### We provide a default extractROWS() method for Vector objects. It calls
-### the extractROWS() generic internally to subset all the "vertical slots".
+### the extractROWS() generic internally to subset all the "parallel slots".
 ### It behaves like an endomorphism with respect to 'x'.
 ### NOTE TO THE DEVELOPERS OF Vector SUBCLASSES: The default extractROWS()
 ### method below will work out-of-the-box and do the right thing on your
-### objects as long as calling vertical_slot_names() on them reports all
-### the "vertical slots". So please make sure to register all the parallel
-### slots via a vertical_slot_names() method.
+### objects as long as calling parallel_slot_names() on them reports all
+### the "parallel slots". So please make sure to register all the parallel
+### slots via a parallel_slot_names() method.
 ### If that simple approach does not work for your objects (typically
 ### because some slots require special treatment) then you should override
 ### the extractROWS() method for Vector objects (you should never need to
@@ -477,15 +504,15 @@ setMethod("extractROWS", "Vector",
         ## instances).
         x <- updateObject(x, check=FALSE)
         i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
-        x_vslotnames <- vertical_slot_names(x)
-        ans_vslots <- lapply(setNames(x_vslotnames, x_vslotnames),
+        x_pslotnames <- parallel_slot_names(x)
+        ans_pslots <- lapply(setNames(x_pslotnames, x_pslotnames),
                              function(slotname)
                                  extractROWS(slot(x, slotname), i))
         ## Does NOT validate the object before returning it, because, most of
         ## the times, this is not needed. There are exceptions though. See
         ## for example the "extractROWS" method for Hits objects.
         do.call(BiocGenerics:::replaceSlots,
-                c(list(x), ans_vslots, list(check=FALSE)))
+                c(list(x), ans_pslots, list(check=FALSE)))
     }
 )
 
@@ -656,7 +683,6 @@ setMethod("summary", "Vector", summary.Vector)
 ### "[<-") work out-of-the-box!
 ###
 
-### Somewhat painful that we do not always have a DataFrame in elementMetadata
 ensureMcols <- function(x) {
   ans <- mcols(x, use.names=FALSE)
   if (is.null(ans))
@@ -688,14 +714,14 @@ rbind_mcols <- function(...)
 }
 
 ### We provide a default bindROWS() method for Vector objects. It calls the
-### bindROWS() generic internally to concatenate all the "vertical slots"
+### bindROWS() generic internally to concatenate all the "parallel slots"
 ### from all the input objects. It behaves like an endomorphism with respect
 ### to its first input object 'x'.
 ### NOTE TO THE DEVELOPERS OF Vector SUBCLASSES: The default bindROWS()
 ### method below will work out-of-the-box and do the right thing on your
-### objects as long as calling vertical_slot_names() on them reports all
-### the "vertical slots". So please make sure to register all the parallel
-### slots via a vertical_slot_names() method.
+### objects as long as calling parallel_slot_names() on them reports all
+### the "parallel slots". So please make sure to register all the parallel
+### slots via a parallel_slot_names() method.
 ### If that simple approach does not work for your objects (typically
 ### because some slots require special treatment) then you should override
 ### the bindROWS() method for Vector objects (you should never need to
@@ -718,9 +744,9 @@ concatenate_Vector_objects <-
     all_objects <- c(list(x), objects)
 
     ## Concatenate all the parallel slots except "NAMES" and "elementMetadata".
-    x_vslotnames <- vertical_slot_names(x)
-    vslotnames <- setdiff(x_vslotnames, c("NAMES", "elementMetadata"))
-    ans_vslots <- lapply(setNames(vslotnames, vslotnames),
+    x_pslotnames <- parallel_slot_names(x)
+    pslotnames <- setdiff(x_pslotnames, c("NAMES", "elementMetadata"))
+    ans_pslots <- lapply(setNames(pslotnames, pslotnames),
         function(slotname) {
             x_slot <- slot(x, slotname)
             if (is.null(x_slot))
@@ -730,7 +756,7 @@ concatenate_Vector_objects <-
         }
     )
 
-    if ("NAMES" %in% x_vslotnames) {
+    if ("NAMES" %in% x_pslotnames) {
         ans_NAMES <- NULL
         if (use.names) {
             names_list <- lapply(all_objects, slot, "NAMES")
@@ -743,17 +769,17 @@ concatenate_Vector_objects <-
                 ans_NAMES <- unlist(names_list, use.names=FALSE)
             }
         }
-        ans_vslots <- c(ans_vslots, list(NAMES=ans_NAMES))
+        ans_pslots <- c(ans_pslots, list(NAMES=ans_NAMES))
     }
 
     if (!ignore.mcols) {
         ## Concatenate the "elementMetadata" slots.
         ans_mcols <- do.call(rbind_mcols, all_objects)
-        ans_vslots <- c(ans_vslots, list(elementMetadata=ans_mcols))
+        ans_pslots <- c(ans_pslots, list(elementMetadata=ans_mcols))
     }
 
     ans <- do.call(BiocGenerics:::replaceSlots,
-                   c(list(x), ans_vslots, list(check=FALSE)))
+                   c(list(x), ans_pslots, list(check=FALSE)))
 
     if (ignore.mcols)
         mcols(ans) <- NULL
@@ -779,8 +805,8 @@ setMethod("c", "Vector",
     }
 )
 
-### FIXME: This method doesn't work properly on DataTable objects if 'after'
-### is >= 1 and < length(x).
+### FIXME: This method doesn't work properly on DataFrame derivatives
+### if 'after' is >= 1 and < length(x).
 setMethod("append", c("Vector", "Vector"),
     function(x, values, after=length(x))
     {
@@ -816,6 +842,16 @@ setMethod("with", "Vector",
             safeEval(substitute(expr), data, parent.frame(), ...)
           })
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### transform()
+###
+
+### NOT exported.
+setGeneric("column<-",
+           function(x, name, value) standardGeneric("column<-"),
+           signature="x")
+
 setReplaceMethod("column", "Vector", function(x, name, value) {
     if (name %in% parallelVectorNames(x)) {
         setter <- get(paste0(name, "<-"), classNamespace(x), mode="function")
@@ -826,8 +862,22 @@ setReplaceMethod("column", "Vector", function(x, name, value) {
     }
 })
 
-transform.Vector <- transformColumns
+transformColumns <- function(`_data`, ...) {
+    exprs <- as.list(substitute(list(...))[-1L])
+    if (any(names(exprs) == "")) {
+        stop("all arguments in '...' must be named")
+    }
+    ## elements in '...' can originate from different environments
+    env <- setNames(top_prenv_dots(...), names(exprs))
+    for (colName in names(exprs)) { # for loop allows inter-arg dependencies
+        value <- safeEval(exprs[[colName]], `_data`, env[[colName]])
+        column(`_data`, colName) <- value
+    }
+    `_data`
+}
 
+### S3/S4 combo for transform.Vector
+transform.Vector <- transformColumns
 setMethod("transform", "Vector", transform.Vector)
 
 
