@@ -18,7 +18,7 @@
 ### the method for vector) for more mess.
 ###
 ### FIXME: Fix the aggregate() mess. Before fixing, it would be good to
-### simplify by gettting rid of the 'frequency' and 'delta' arguments.
+### simplify by getting rid of the 'frequency' and 'delta' arguments.
 ### Then the 'start', 'end', and 'width' arguments wouldn't be needed
 ### anymore because the user can aggregate by range by passing
 ### IRanges(start, end, width) to 'by'. After removing these arguments,
@@ -190,7 +190,7 @@ aggregateWithDots <- function(x, by, FUN, ..., drop = TRUE) {
 
     endomorphism <- FALSE
     if (missing(by)) {
-        if (is(x, "List") && !is(x, "DataTable") && !is(x, "Ranges")) {
+        if (is(x, "List") && !is(x, "DataFrame") && !is(x, "Ranges")) {
             by <- IRanges::PartitioningByEnd(x)
             x <- unlist(x, use.names=FALSE)
         } else {
@@ -205,7 +205,7 @@ aggregateWithDots <- function(x, by, FUN, ..., drop = TRUE) {
     
     if (is(by, "formula")) {
         by <- ModelFrame(by, x)
-    } else if (is.list(by) || is(by, "DataTable")) {
+    } else if (is.list(by) || is(by, "DataFrame")) {
         by <- IRanges::FactorList(by, compress=FALSE)
     }
     
