@@ -293,7 +293,7 @@ setMethod("classNameForDisplay", "AsIs",
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### showAsCell()
 ###
-### All "showAsCell" methods should return a character vector.
+### All "showAsCell" methods must return a character vector.
 ###
 
 ### Exported!
@@ -335,8 +335,10 @@ showAsCell_array <- function(object)
         return(character(0L))
     attempt <- try(as.character(object), silent=TRUE)
     if (!is(attempt, "try-error"))
-        return(attempt)
-    rep.int("#####", object_NROW)
+        return(showAsCell(attempt))
+    if (object_NROW == 1L)
+        return(paste0("<", classNameForDisplay(object), ">"))
+    rep.int("####", object_NROW)
 }
 
 setMethod("showAsCell", "ANY", .default_showAsCell)
@@ -350,13 +352,47 @@ setMethod("showAsCell", "numeric",
     }
 )
 
+setMethod("showAsCell", "character",
+    function(object)
+    {
+        nc <- nchar(object, type="chars")
+        trim_idx <- which(nc > 22L)
+        if (length(trim_idx) != 0L) {
+            tmp <- substr(object[trim_idx], start=1L, stop=20L)
+            object[trim_idx] <- paste0(tmp, "..")
+        }
+        object
+    }
+)
+
 setMethod("showAsCell", "AsIs",
     function(object) showAsCell(drop_AsIs(object))
 )
 
-### Mmmh... these methods don't return a character vector. Is that ok?
-setMethod("showAsCell", "Date", function(object) object)
-setMethod("showAsCell", "POSIXt", function(object) object)
+showAsCell_list <- function(object)
+{
+    vapply(object,
+        function(x) {
+            ## 'x' is not necessarily subsettable so if its length is 1 (e.g.
+            ## 'x' is a BamFile object) we avoid the risky subsetting.
+            if (NROW(x) == 1L)
+                return(showAsCell(x))
+            ## 'head(x, 3L)' is still no guaranteed to work.
+            x3 <- try(head(x, 3L), silent=TRUE)
+            if (is(x3, "try-error"))
+                return("####")
+            str <- paste(showAsCell(x3), collapse=",")
+            if (length(x) > 3L)
+                str <- paste0(str, ",...")
+            str
+        },
+        character(1L),
+        USE.NAMES=FALSE
+    )
+}
+setMethod("showAsCell", "list", showAsCell_list)
+
+setMethod("showAsCell", "data.frame", showAsCell_array)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -468,4 +504,3 @@ makeClassinfoRowForCompactPrinting <- function(x, col2class)
     matrix(ans, nrow=1L, dimnames=list("", names(ans)))
 }
 
-
